@@ -2,15 +2,13 @@
 
 from __future__ import print_function
 
-from pydoctor import model, zopeinterface
-from pydoctor.sphinx import (
-    SphinxInventory,
-    prepareCache,
-    MAX_AGE_HELP,
-    USER_INTERSPHINX_CACHE
-)
+import datetime
+import os
+import sys
 
-import sys, os, datetime
+from pydoctor import model, zopeinterface, __version__
+from pydoctor.sphinx import (MAX_AGE_HELP, USER_INTERSPHINX_CACHE,
+                             SphinxInventoryWriter, prepareCache)
 
 BUILDTIME_FORMAT = '%Y-%m-%d %H:%M:%S'
 
@@ -38,24 +36,11 @@ MAKE_HTML_DEFAULT = object()
 
 def getparser():
     from optparse import OptionParser
-    parser = OptionParser()
+    parser = OptionParser(version=__version__.public())
     parser.add_option(
         '-c', '--config', dest='configfile',
         help=("Use config from this file (any command line"
               "options override settings from the file)."))
-    parser.add_option(
-        '-p', '--input-pickle', dest='inputpickle',
-        help=("Load the system from this pickle file (default: "
-              "none, a blank system is created)."))
-    parser.add_option(
-        '-o', '--output-pickle', dest='outputpickle',
-        help=("Save the system to this pickle file (default: "
-              "none, the system is not saved by default)."))
-    parser.add_option(
-        '--extra-system', action='append', dest='moresystems',
-        metavar='SYS:URLPREFIX', default=[],
-        help=("Look for objects in this system to.  Links to these "
-              "objects will have URLPREFIX prepended to them."))
     parser.add_option(
         '--system-class', dest='systemclass',
         help=("A dotted name of the class to use to make a system."))
@@ -227,8 +212,7 @@ def parse_args(args):
     options.verbosity -= options.quietness
     return options, args
 
-def main(args):
-    from six.moves import cPickle
+def main(args=sys.argv[1:]):
     options, args = parse_args(args)
 
     exitcode = 0
@@ -252,34 +236,14 @@ def main(args):
         else:
             systemclass = zopeinterface.ZopeInterfaceSystem
 
-        if options.inputpickle:
-            system = cPickle.load(open(options.inputpickle, 'rb'))
-            if options.systemclass:
-                if type(system) is not systemclass:
-                    cls = type(system)
-                    msg = ("loaded pickle has class %s.%s, differing "
-                           "from explicitly requested %s")
-                    error(msg, cls.__module__, cls.__name__,
-                          options.systemclass)
-        else:
-            system = systemclass()
-
-        # Once pickle support is removed, always instantiate System with
-        # options and make fetchIntersphinxInventories private in __init__.
-        system.options = options
+        system = systemclass(options)
         system.fetchIntersphinxInventories(cache)
 
-        system.urlprefix = ''
-        if options.moresystems:
-            moresystems = []
-            for fnamepref in options.moresystems:
-                fname, prefix = fnamepref.split(':', 1)
-                moresystems.append(cPickle.load(open(fname, 'rb')))
-                moresystems[-1].urlprefix = prefix
-                moresystems[-1].options = system.options
-                moresystems[-1].subsystems.append(system)
-            system.moresystems = moresystems
-        system.sourcebase = options.htmlsourcebase
+        if options.htmlsourcebase:
+            if options.projectbasedirectory is None:
+                error("you must specify --project-base-dir "
+                      "when using --html-viewsource-base")
+            system.sourcebase = options.htmlsourcebase
 
         if options.abbrevmapping:
             for thing in options.abbrevmapping.split(','):
@@ -290,8 +254,7 @@ def main(args):
         args = list(args) + options.modules + options.packages
 
         if options.makehtml == MAKE_HTML_DEFAULT:
-            if not options.outputpickle and not options.testing \
-                   and not options.makeintersphinx:
+            if not options.testing and not options.makeintersphinx:
                 options.makehtml = True
             else:
                 options.makehtml = False
@@ -320,9 +283,9 @@ def main(args):
             if options.prependedpackage:
                 for m in options.prependedpackage.split('.'):
                     prependedpackage = system.Package(
-                        system, m, None, prependedpackage)
+                        system, m, prependedpackage)
                     system.addObject(prependedpackage)
-                    initmodule = system.Module(system, '__init__', None, prependedpackage)
+                    initmodule = system.Module(system, '__init__', prependedpackage)
                     system.addObject(initmodule)
             for path in args:
                 path = os.path.abspath(path)
@@ -353,17 +316,7 @@ def main(args):
         else:
             system.projectname = system.options.projectname
 
-        # step 4: save the system, if desired
-
-        if options.outputpickle:
-            system.msg('', 'saving output pickle to ' + options.outputpickle)
-            del system.options # don't persist the options
-            f = open(options.outputpickle, 'wb')
-            cPickle.dump(system, f, cPickle.HIGHEST_PROTOCOL)
-            f.close()
-            system.options = options
-
-        # step 5: make html, if desired
+        # step 4: make html, if desired
 
         if options.makehtml:
             options.makeintersphinx = True
@@ -382,8 +335,6 @@ def main(args):
             writer.system = system
             writer.prepOutputDirectory()
 
-            system.epytextproblems = []
-
             if options.htmlsubjects:
                 subjects = []
                 for fn in options.htmlsubjects:
@@ -395,29 +346,20 @@ def main(args):
                 writer.writeModuleIndex(system)
                 subjects = system.rootobjects
             writer.writeIndividualFiles(subjects, options.htmlfunctionpages)
-            if system.epytextproblems:
+            if system.docstring_syntax_errors:
                 def p(msg):
                     system.msg(('epytext', 'epytext-summary'), msg, thresh=-1, topthresh=1)
-                p("these %s objects' docstrings are not proper epytext:"
-                  %(len(system.epytextproblems),))
+                p("these %s objects' docstrings contain syntax errors:"
+                  %(len(system.docstring_syntax_errors),))
                 exitcode = 2
-                for fn in system.epytextproblems:
+                for fn in sorted(system.docstring_syntax_errors):
                     p('    '+fn)
-            if options.outputpickle:
-                system.msg(
-                    '', 'saving output pickle to ' + options.outputpickle)
-                # save again, with epytextproblems
-                del system.options # don't persist the options
-                f = open(options.outputpickle, 'wb')
-                cPickle.dump(system, f, cPickle.HIGHEST_PROTOCOL)
-                f.close()
-                system.options = options
 
         if options.makeintersphinx:
             if not options.makehtml:
                 subjects = system.rootobjects
             # Generate Sphinx inventory.
-            sphinx_inventory = SphinxInventory(
+            sphinx_inventory = SphinxInventoryWriter(
                 logger=system.msg,
                 project_name=system.projectname,
                 )
@@ -433,6 +375,3 @@ def main(args):
             pdb.post_mortem(sys.exc_info()[2])
         raise
     return exitcode
-
-if __name__ == '__main__':
-    sys.exit(main(sys.argv[1:]))
