@@ -10,30 +10,22 @@ Markup language support for docstrings.  Each submodule defines a
 parser for a single markup language.  These parsers convert an
 object's docstring to a L{ParsedDocstring}, a standard intermediate
 representation that can be used to generate output.
-C{ParsedDocstring}s support the following operations:
-  - output generation (L{to_html()<ParsedDocstring.to_html>}).
-  - Field extraction (L{split_fields()<ParsedDocstring.split_fields>}).
 
-The L{parse()} function provides a single interface to the
-C{pydoctor.epydoc.markup} package: it takes a docstring and the name of
-a markup language; delegates to the appropriate parser; and returns the
-parsed docstring (along with any errors or warnings that were
-generated).
+A C{ParsedDocstring} is used for output generation
+(L{to_stan()<ParsedDocstring.to_stan>}).
+It also stores the fields that were extracted from the docstring
+during parsing (L{fields<ParsedDocstring.fields>}).
 
-The C{ParsedDocstring} output generation methods (C{to_M{format}()})
-use a L{DocstringLinker} to link the docstring output with the rest of
+The C{parse_docstring()} functions in the format modules take a docstring,
+parse it and return a format-specific subclass of C{ParsedDocstring}.
+A docstring's fields are separated from the body during parsing.
+
+The C{ParsedDocstring} output generation method
+(L{to_stan()<ParsedDocstring.to_stan>}) uses a
+L{DocstringLinker} to link the docstring output with the rest of
 the documentation that epydoc generates.  C{DocstringLinker}s are
-currently responsible for translating two kinds of crossreference:
-  - index terms (L{translate_indexterm()
-    <DocstringLinker.translate_indexterm>}).
-  - identifier crossreferences (L{translate_identifier_xref()
-    <DocstringLinker.translate_identifier_xref>}).
-
-A parsed docstring's fields can be extracted using the
-L{ParsedDocstring.split_fields()} method.  This method divides a
-docstring into its main body and a list of L{Field}s, each of which
-encodes a single field.  The field's bodies are encoded as
-C{ParsedDocstring}s.
+responsible for resolving identifier crossreferences
+(L{resolve_identifier_xref() <DocstringLinker.resolve_identifier_xref>}).
 
 Markup errors are represented using L{ParseError}s.  These exception
 classes record information about the cause, location, and severity of
@@ -44,6 +36,11 @@ each error.
 """
 __docformat__ = 'epytext en'
 
+import re
+
+from six import text_type
+from twisted.web.template import XMLString, flattenString
+
 ##################################################
 ## Contents
 ##################################################
@@ -61,45 +58,72 @@ class ParsedDocstring:
     """
     A standard intermediate representation for parsed docstrings that
     can be used to generate output.  Parsed docstrings are produced by
-    markup parsers (such as L{epytext.parse} or L{javadoc.parse}).
-    C{ParsedDocstring}s support several kinds of operation:
-      - output generation (L{to_html()}).
-      - Field extraction (L{split_fields()}).
+    markup parsers such as L{pydoctor.epydoc.epytext.parse_docstring()}
+    or L{pydoctor.epydoc.restructuredtext.parse_docstring()}.
 
-    The output generation methods (C{to_M{format}()}) use a
-    L{DocstringLinker} to link the docstring output with the rest
-    of the documentation that epydoc generates.
-
-    Subclasses must implement all methods of this class.
+    Subclasses must implement L{to_stan()}.
     """
 
-    def split_fields(self, errors=None):
+    def __init__(self, fields):
+        self.fields = fields
+        """
+        A list of L{Field}s, each of which encodes a single field.
+        The field's bodies are encoded as C{ParsedDocstring}s.
         """
-        Split this docstring into its body and its fields.
 
-        @return: A tuple C{(M{body}, M{fields})}, where C{M{body}} is
-            the main body of this docstring, and C{M{fields}} is a list
-            of its fields.  If the resulting body is empty, return
-            C{None} for the body.
-        @rtype: C{(L{ParsedDocstring}, list of L{Field})}
-        @param errors: A list where any errors generated during
-            splitting will be stored.  If no list is specified, then
-            errors will be ignored.
-        @type errors: C{list} of L{ParseError}
+    def to_stan(self, docstring_linker):
         """
-        raise NotImplementedError('ParsedDocstring.split_fields()')
+        Translate this docstring to a Stan tree.
 
-    def to_html(self, docstring_linker):
-        """
-        Translate this docstring to HTML.
+        Implementations are encouraged to generate Stan output directly
+        if possible, but if that is not feasible, L{html2stan()} can be
+        used instead.
 
         @param docstring_linker: An HTML translator for crossreference
             links into and out of the docstring.
         @type docstring_linker: L{DocstringLinker}
-        @return: An HTML fragment that encodes this docstring.
-        @rtype: C{string}
+        @return: The docstring presented as a tree.
         """
-        raise NotImplementedError('ParsedDocstring.to_html()')
+        raise NotImplementedError()
+
+_RE_CONTROL = re.compile((
+    '[' + ''.join(
+    ch for ch in map(chr, range(0, 32)) if ch not in '\r\n\t\f'
+    ) + ']'
+    ).encode())
+
+def html2stan(html):
+    """
+    Convert an HTML string to a Stan tree.
+
+    @param html: An HTML fragment; multiple roots are allowed.
+    @type html: C{string}
+    @return: The fragment as a tree with a transparent root node.
+    """
+    if isinstance(html, text_type):
+        html = html.encode('utf8')
+
+    html = _RE_CONTROL.sub(lambda m:b'\\x%02x' % ord(m.group()), html)
+    stan = XMLString(b'<div>%s</div>' % html).load()[0]
+    assert stan.tagName == 'div'
+    stan.tagName = ''
+    return stan
+
+def flatten(stan):
+    """
+    Convert a document fragment from a Stan tree to HTML.
+
+    @param stan: Document fragment to flatten.
+    @return: An HTML string representation of the C{stan} tree.
+    @rtype: C{str}
+    """
+    ret = []
+    err = []
+    flattenString(None, stan).addCallback(ret.append).addErrback(err.append)
+    if err:
+        raise err[0].value
+    else:
+        return ret[0].decode()
 
 ##################################################
 ## Fields
@@ -119,11 +143,12 @@ class Field:
     Tags are automatically downcased and stripped; and arguments are
     automatically stripped.
     """
-    def __init__(self, tag, arg, body):
+    def __init__(self, tag, arg, body, lineno):
         self._tag = tag.lower().strip()
         if arg is None: self._arg = None
         else: self._arg = arg.strip()
         self._body = body
+        self.lineno = lineno
 
     def tag(self):
         """
@@ -158,42 +183,23 @@ class Field:
 ##################################################
 class DocstringLinker:
     """
-    A translator for crossreference links into and out of a
-    C{ParsedDocstring}.  C{DocstringLinker} is used by
-    C{ParsedDocstring} to convert these crossreference links into
-    appropriate output formats.  For example,
-    C{DocstringLinker.to_html} expects a C{DocstringLinker} that
-    converts crossreference links to HTML.
+    A resolver for crossreference links out of a C{ParsedDocstring}.
+    C{DocstringLinker} is used by C{ParsedDocstring} to look up the
+    target URL for crossreference links.
     """
-    def translate_indexterm(self, indexterm):
-        """
-        Translate an index term to the appropriate output format.  The
-        output will typically include a crossreference anchor.
 
-        @type indexterm: L{ParsedDocstring}
-        @param indexterm: The index term to translate.
-        @rtype: C{string}
-        @return: The translated index term.
+    def resolve_identifier_xref(self, identifier):
         """
-        raise NotImplementedError('DocstringLinker.translate_indexterm()')
-
-    def translate_identifier_xref(self, identifier, label=None):
-        """
-        Translate a crossreference link to a Python identifier to the
-        appropriate output format.  The output will typically include
-        a reference or pointer to the crossreference target.
+        Resolve a crossreference link to a Python identifier.
 
         @type identifier: C{string}
         @param identifier: The name of the Python identifier that
             should be linked to.
-        @type label: C{string} or C{None}
-        @param label: The label that should be used for the identifier,
-            if it's different from the name of the identifier.  This
-            should be expressed in the target markup language.
         @rtype: C{string}
-        @return: The translated crossreference link.
+        @return: The URL of the target
+        @raise LookupError: If C{identifier} could not be resolved.
         """
-        raise NotImplementedError('DocstringLinker.translate_xref()')
+        raise NotImplementedError()
 
 
 ##################################################
@@ -275,17 +281,3 @@ class ParseError(Exception):
             return '<ParseError on unknown line>'
         else:
             return '<ParseError on line %d>' % (self._linenum + 1)
-
-    def __cmp__(self, other):
-        """
-        Compare two C{ParseError}s, based on their line number.
-          - Return -1 if C{self.linenum<other.linenum}
-          - Return +1 if C{self.linenum>other.linenum}
-          - Return 0 if C{self.linenum==other.linenum}.
-        The return value is undefined if C{other} is not a
-        ParseError.
-
-        @rtype: C{int}
-        """
-        if not isinstance(other, ParseError): return -1000
-        return cmp(self._linenum, other._linenum)
