@@ -6,17 +6,20 @@ system being documented.  An instance of L{System} represents the whole system
 being documented -- a System is a bad of Documentables, in some sense.
 """
 
-from __future__ import print_function
+from __future__ import print_function, unicode_literals
 
 import datetime
 import imp
+import inspect
 import os
-import posixpath
+import platform
 import sys
 import types
-from six.moves import builtins
+from collections import OrderedDict
+from enum import Enum
 
 from pydoctor.sphinx import SphinxInventory
+from six.moves import builtins
 
 # originally when I started to write pydoctor I had this idea of a big
 # tree of Documentables arranged in an almost arbitrary tree.
@@ -32,7 +35,14 @@ from pydoctor.sphinx import SphinxInventory
 #   Functions can't contain anything.
 
 
-class DocLocation:
+_string_lineno_is_end = sys.version_info < (3,8) \
+                    and platform.python_implementation() != 'PyPy'
+"""True iff the 'lineno' attribute of an AST string node points to the last
+line in the string, rather than the first line.
+"""
+
+
+class DocLocation(Enum):
     OWN_PAGE = 1
     PARENT_PAGE = 2
     # Nothing uses this yet.  Parameters will one day.
@@ -54,20 +64,22 @@ class Documentable(object):
     @ivar kind: ...
     """
     documentation_location = DocLocation.OWN_PAGE
+    docstring = None
+    docstring_lineno = 0
+    linenumber = 0
     sourceHref = None
 
     @property
     def css_class(self):
         """A short, lower case description for use as a CSS class in HTML."""
         class_ = self.kind.lower().replace(' ', '')
-        if self.privacyClass == PrivacyClass.PRIVATE:
+        if self.privacyClass is PrivacyClass.PRIVATE:
             class_ += ' private'
         return class_
 
-    def __init__(self, system, name, docstring, parent=None):
+    def __init__(self, system, name, parent=None):
         self.system = system
         self.name = name
-        self.docstring = docstring
         self.parent = parent
         self.parentMod = None
         self.setup()
@@ -75,8 +87,37 @@ class Documentable(object):
             self.doctarget = self
 
     def setup(self):
-        self.contents = {}
-        self.orderedcontents = []
+        self.contents = OrderedDict()
+
+    def setDocstring(self, node):
+        doc = node.s
+        lineno = node.lineno
+        if _string_lineno_is_end:
+            # In older CPython versions, the AST only tells us the end line
+            # number and we must approximate the start line number.
+            # This approximation is correct if the docstring does not contain
+            # explicit newlines ('\n') or joined lines ('\' at end of line).
+            lineno -= doc.count('\n')
+
+        # Leading blank lines are stripped by cleandoc(), so we must
+        # return the line number of the first non-blank line.
+        for ch in doc:
+            if ch == '\n':
+                lineno += 1
+            elif not ch.isspace():
+                break
+
+        self.docstring = inspect.cleandoc(doc)
+        self.docstring_lineno = lineno
+
+    def setLineNumber(self, lineno):
+        if not self.linenumber:
+            self.linenumber = lineno
+            parentMod = self.parentMod
+            if parentMod is not None:
+                parentSourceHref = parentMod.sourceHref
+                if parentSourceHref:
+                    self.sourceHref = '%s#L%d' % (parentSourceHref, lineno)
 
     def fullName(self):
         parent = self.parent
@@ -116,20 +157,18 @@ class Documentable(object):
         self.name = new_name
         self._handle_reparenting_post()
         del old_parent.contents[old_name]
-        old_parent.orderedcontents.remove(self)
         old_parent._localNameToFullName_map[old_name] = self.fullName()
         new_parent.contents[new_name] = self
-        new_parent.orderedcontents.append(self)
         self._handle_reparenting_post()
 
     def _handle_reparenting_pre(self):
         del self.system.allobjects[self.fullName()]
-        for o in self.orderedcontents:
+        for o in self.contents.values():
             o._handle_reparenting_pre()
 
     def _handle_reparenting_post(self):
         self.system.allobjects[self.fullName()] = self
-        for o in self.orderedcontents:
+        for o in self.contents.values():
             o._handle_reparenting_post()
 
     def _localNameToFullName(self, name):
@@ -185,41 +224,37 @@ class Documentable(object):
 
         This is just a simple helper which defers to self.privacyClass.
         """
-        return self.privacyClass != PrivacyClass.HIDDEN
+        return self.privacyClass is not PrivacyClass.HIDDEN
 
-    def __getstate__(self):
-        # this is so very, very evil.
-        # see doc/extreme-pickling-pain.txt for more.
-        r = {}
-        for k, v in self.__dict__.iteritems():
-            if isinstance(v, Documentable):
-                r['$'+k] = v.fullName()
-            elif isinstance(v, list) and v:
-                for vv in v:
-                    if vv is not None and not isinstance(vv, Documentable):
-                        r[k] = v
-                        break
-                else:
-                    rr = []
-                    for vv in v:
-                        if vv is None:
-                            rr.append(vv)
-                        else:
-                            rr.append(vv.fullName())
-                    r['@'+k] = rr
-            elif isinstance(v, dict) and v:
-                for vv in v.itervalues():
-                    if not isinstance(vv, Documentable):
-                        r[k] = v
-                        break
-                else:
-                    rr = {}
-                    for kk, vv in v.iteritems():
-                        rr[kk] = vv.fullName()
-                    r['!'+k] = rr
-            else:
-                r[k] = v
-        return r
+    @property
+    def module(self):
+        """This object's L{Module}.
+
+        For modules, this returns the object itself, otherwise
+        the module containing the object is returned.
+        """
+        parentMod = self.parentMod
+        assert parentMod is not None
+        return parentMod
+
+    def report(self, descr, section='parsing', lineno_offset=0):
+        """Log an error or warning about this documentable object."""
+
+        if section == 'docstring':
+            linenumber = self.docstring_lineno
+        else:
+            linenumber = self.linenumber
+        if linenumber:
+            linenumber += lineno_offset
+        elif lineno_offset and self.module is self:
+            linenumber = lineno_offset
+        else:
+            linenumber = '???'
+
+        self.system.msg(
+            section,
+            '%s:%s: %s' % (self.module.description, linenumber, descr),
+            thresh=-1)
 
 
 class Package(Documentable):
@@ -230,6 +265,9 @@ class Package(Documentable):
     def doctarget(self):
         return self.contents['__init__']
     @property
+    def module(self):
+        return self.contents['__init__']
+    @property
     def state(self):
         return self.contents['__init__'].state
 
@@ -241,7 +279,11 @@ class Package(Documentable):
             return self.contents['__init__']._localNameToFullName(name)
 
 
-[UNPROCESSED, PROCESSING, PROCESSED] = range(3)
+class ProcessingState(Enum):
+    UNPROCESSED = 0
+    PROCESSING = 1
+    PROCESSED = 2
+
 
 class CanContainImportsDocumentable(Documentable):
     def setup(self):
@@ -251,8 +293,8 @@ class CanContainImportsDocumentable(Documentable):
 
 class Module(CanContainImportsDocumentable):
     kind = "Module"
-    state = UNPROCESSED
-    linenumber = 0
+    state = ProcessingState.UNPROCESSED
+
     def setup(self):
         super(Module, self).setup()
         self.all = None
@@ -268,6 +310,21 @@ class Module(CanContainImportsDocumentable):
         else:
             return name
 
+    @property
+    def module(self):
+        return self
+
+    @property
+    def description(self):
+        """A string describing this module to the user.
+
+        If this module's code was read from a file, this returns
+        its file path. In other cases, such as during unit testing,
+        the module's full name is returned.
+        """
+        filepath = getattr(self, 'filepath', None)
+        return self.fullName() if filepath is None else filepath
+
 
 class Class(CanContainImportsDocumentable):
     kind = "Class"
@@ -294,14 +351,9 @@ class Class(CanContainImportsDocumentable):
             return self.parent._localNameToFullName(name)
 
 
-class Function(Documentable):
+class Inheritable(Documentable):
     documentation_location = DocLocation.PARENT_PAGE
-    kind = "Function"
-    linenumber = 0
-    def setup(self):
-        super(Function, self).setup()
-        if isinstance(self.parent, Class):
-            self.kind = "Method"
+
     def docsources(self):
         yield self
         if not isinstance(self.parent, Class):
@@ -309,20 +361,24 @@ class Function(Documentable):
         for b in self.parent.allbases(include_self=False):
             if self.name in b.contents:
                 yield b.contents[self.name]
+
     def _localNameToFullName(self, name):
         return self.parent._localNameToFullName(name)
 
-class Attribute(Documentable):
+class Function(Inheritable):
+    kind = "Function"
 
-    linenumber = 0
+    def setup(self):
+        super(Function, self).setup()
+        if isinstance(self.parent, Class):
+            self.kind = "Method"
+
+class Attribute(Inheritable):
     kind = "Attribute"
-    documentation_location = DocLocation.PARENT_PAGE
 
-    def _localNameToFullName(self, name):
-        return self.parent._localNameToFullName(name)
 
-class PrivacyClass:
-    """'enum' containing values indicating how private an object should be.
+class PrivacyClass(Enum):
+    """L{Enum} containing values indicating how private an object should be.
 
     @cvar HIDDEN: Don't show the object at all.
     @cvar PRIVATE: Show, but de-emphasize the object.
@@ -352,14 +408,10 @@ class System(object):
     sourcebase = None
 
     def __init__(self, options=None):
-        self.allobjects = {}
-        self.orderedallobjects = []
+        self.allobjects = OrderedDict()
         self.rootobjects = []
         self.warnings = {}
         self.packages = []
-        self.moresystems = []
-        self.subsystems = []
-        self.urlprefix = ''
 
         if options:
             self.options = options
@@ -370,7 +422,10 @@ class System(object):
 
         self.abbrevmapping = {}
         self.projectname = 'my project'
-        self.epytextproblems = [] # fullNames of objects that failed to epytext properly
+
+        self.docstring_syntax_errors = set()
+        """FullNames of objects for which the docstring failed to parse."""
+
         self.verboselevel = 0
         self.needsnl = False
         self.once_msgs = set()
@@ -378,10 +433,7 @@ class System(object):
         self.module_count = 0
         self.processing_modules = []
         self.buildtime = datetime.datetime.now()
-        # Once pickle support is removed, System should be
-        # initialized with project name so that we can reuse intersphinx instance for
-        # object.inv generation.
-        self.intersphinx = SphinxInventory(logger=self.msg, project_name=self.projectname)
+        self.intersphinx = SphinxInventory(logger=self.msg)
 
     def verbosity(self, section=None):
         if isinstance(section, str):
@@ -418,13 +470,10 @@ class System(object):
                 sys.stdout.flush()
             else:
                 self.needsnl = False
-                print()
+                print('')
 
     def objForFullName(self, fullName):
-        for system in [self] + self.moresystems:
-            if fullName in system.allobjects:
-                return system.allobjects[fullName]
-        return None
+        return self.allobjects.get(fullName)
 
     def _warning(self, current, type, detail):
         if current is not None:
@@ -437,70 +486,29 @@ class System(object):
 
     def objectsOfType(self, cls):
         """Iterate over all instances of C{cls} present in the system. """
-        for o in self.orderedallobjects:
+        for o in self.allobjects.values():
             if isinstance(o, cls):
                 yield o
 
     def privacyClass(self, ob):
+        if ob.kind is None:
+            return PrivacyClass.HIDDEN
         if ob.name.startswith('_') and \
                not (ob.name.startswith('__') and ob.name.endswith('__')):
             return PrivacyClass.PRIVATE
         return PrivacyClass.VISIBLE
 
-    def __getstate__(self):
-        d = self.__dict__.copy()
-        del d['intersphinx']
-        return d
-
-    def __setstate__(self, state):
-        if 'abbrevmapping' not in state:
-            state['abbrevmapping'] = {}
-        # this is so very, very evil.
-        # see doc/extreme-pickling-pain.txt for more.
-        def lookup(name):
-            for system in [self] + self.moresystems + self.subsystems:
-                if name in system.allobjects:
-                    return system.allobjects[name]
-            raise KeyError(name)
-        self.__dict__.update(state)
-        for system in [self] + self.moresystems + self.subsystems:
-            if 'allobjects' not in system.__dict__:
-                return
-        for system in [self] + self.moresystems + self.subsystems:
-            for obj in system.orderedallobjects:
-                for k, v in obj.__dict__.copy().iteritems():
-                    if k.startswith('$'):
-                        del obj.__dict__[k]
-                        obj.__dict__[k[1:]] = lookup(v)
-                    elif k.startswith('@'):
-                        n = []
-                        for vv in v:
-                            if vv is None:
-                                n.append(None)
-                            else:
-                                n.append(lookup(vv))
-                        del obj.__dict__[k]
-                        obj.__dict__[k[1:]] = n
-                    elif k.startswith('!'):
-                        n = {}
-                        for kk, vv in v.iteritems():
-                            n[kk] = lookup(vv)
-                        del obj.__dict__[k]
-                        obj.__dict__[k[1:]] = n
-        self.intersphinx = SphinxInventory(logger=self.msg, project_name=self.projectname)
-
     def addObject(self, obj):
         """Add C{object} to the system."""
-        if obj.parent:
-            obj.parent.orderedcontents.append(obj)
+        fullName = obj.fullName()
+        if obj.parent and obj.parent.fullName() != fullName:
             obj.parent.contents[obj.name] = obj
         else:
             self.rootobjects.append(obj)
-        self.orderedallobjects.append(obj)
-        if obj.fullName() in self.allobjects:
+        if fullName in self.allobjects:
             self.handleDuplicate(obj)
         else:
-            self.allobjects[obj.fullName()] = obj
+            self.allobjects[fullName] = obj
 
     # if we assume:
     #
@@ -525,33 +533,17 @@ class System(object):
     def setSourceHref(self, mod):
         if self.sourcebase is None:
             mod.sourceHref = None
-            return
-
-        projBaseDir = mod.system.options.projectbasedirectory
-        if projBaseDir is not None:
+        else:
+            projBaseDir = mod.system.options.projectbasedirectory
             mod.sourceHref = (
                 self.sourcebase +
                 mod.filepath[len(projBaseDir):])
-            return
-
-        trailing = []
-        dir, fname = os.path.split(mod.filepath)
-        while os.path.exists(os.path.join(dir, '.svn')):
-            dir, dirname = os.path.split(dir)
-            trailing.append(dirname)
-
-        # now trailing[-1] would be 'Divmod' in the above example
-        del trailing[-1]
-        trailing.reverse()
-        trailing.append(fname)
-
-        mod.sourceHref = posixpath.join(mod.system.sourcebase, *trailing)
 
     def addModule(self, modpath, modname, parentPackage=None):
-        mod = self.Module(self, modname, None, parentPackage)
+        mod = self.Module(self, modname, parentPackage)
         self.addObject(mod)
         self.progress(
-            "addModule", len(self.orderedallobjects),
+            "addModule", len(self.allobjects),
             None, "modules and packages discovered")
         mod.filepath = modpath
         self.unprocessed_modules.add(mod)
@@ -567,7 +559,7 @@ class System(object):
         else:
             parent_package = None
             module_name = module_full_name
-        module = self.Module(self, module_name, None, parent_package)
+        module = self.Module(self, module_name, parent_package)
         self.addObject(module)
         return module
 
@@ -580,24 +572,27 @@ class System(object):
         else:
             parent_package = None
             package_name = package_full_name
-        package = self.Package(self, package_name, None, parent_package)
+        package = self.Package(self, package_name, parent_package)
         self.addObject(package)
         return package
 
     def _introspectThing(self, thing, parent, parentMod):
-        for k, v in thing.__dict__.iteritems():
-            if isinstance(v, (types.BuiltinFunctionType, type(dict.keys))):
-                f = self.Function(self, k, v.__doc__, parent)
+        for k, v in thing.__dict__.items():
+            if isinstance(v, (types.BuiltinFunctionType,
+                              types.FunctionType)):
+                f = self.Function(self, k, parent)
                 f.parentMod = parentMod
+                f.docstring = v.__doc__
                 f.decorators = None
                 f.argspec = ((), None, None, ())
                 self.addObject(f)
             elif isinstance(v, type):
-                c = self.Class(self, k, v.__doc__, parent)
+                c = self.Class(self, k, parent)
                 c.bases = []
                 c.baseobjects = []
                 c.rawbases = []
                 c.parentMod = parentMod
+                c.docstring = v.__doc__
                 self.addObject(c)
                 self._introspectThing(v, c, parentMod)
 
@@ -605,7 +600,6 @@ class System(object):
         module = self.ensureModule(module_full_name)
         module.docstring = py_mod.__doc__
         self._introspectThing(py_mod, module, module)
-        print(py_mod)
 
     def addPackage(self, dirpath, parentPackage=None):
         if not os.path.exists(dirpath):
@@ -668,24 +662,24 @@ class System(object):
         The default is that the second definition "wins".
         '''
         i = 0
-        fn = obj.fullName()
-        while (fn + ' ' + str(i)) in self.allobjects:
+        fullName = obj.fullName()
+        while (fullName + ' ' + str(i)) in self.allobjects:
             i += 1
-        prev = self.allobjects[obj.fullName()]
+        prev = self.allobjects[fullName]
         self._warning(obj.parent, "duplicate", prev)
         def remove(o):
             del self.allobjects[o.fullName()]
-            for c in o.orderedcontents:
+            oc = list(o.contents.values())
+            for c in oc:
                 remove(c)
         remove(prev)
         prev.name = obj.name + ' ' + str(i)
         def readd(o):
             self.allobjects[o.fullName()] = o
-            for c in o.orderedcontents:
+            for c in o.contents.values():
                 readd(c)
         readd(prev)
-        self.allobjects[obj.fullName()] = obj
-        return obj
+        self.allobjects[fullName] = obj
 
 
     def getProcessedModule(self, modname):
@@ -697,15 +691,15 @@ class System(object):
         if not isinstance(mod, Module):
             return None
 
-        if mod.state == UNPROCESSED:
+        if mod.state is ProcessingState.UNPROCESSED:
             self.processModule(mod)
 
         return mod
 
 
     def processModule(self, mod):
-        assert mod.state == UNPROCESSED
-        mod.state = PROCESSING
+        assert mod.state is ProcessingState.UNPROCESSED
+        mod.state = ProcessingState.PROCESSING
         if getattr(mod, 'filepath', None) is None:
             return
         builder = self.defaultBuilder(self)
@@ -714,7 +708,7 @@ class System(object):
             self.processing_modules.append(mod.fullName())
             self.msg("processModule", "processing %s"%(self.processing_modules), 1)
             builder.processModuleAST(ast, mod)
-            mod.state = PROCESSED
+            mod.state = ProcessingState.PROCESSED
             head = self.processing_modules.pop()
             assert head == mod.fullName()
         self.unprocessed_modules.remove(mod)
@@ -723,7 +717,7 @@ class System(object):
             self.module_count - len(self.unprocessed_modules),
             self.module_count,
             "modules processed %s warnings"%(
-            sum(len(v) for v in self.warnings.itervalues()),))
+            sum(len(v) for v in self.warnings.values()),))
 
 
     def process(self):
