@@ -2,17 +2,19 @@
 
 from __future__ import print_function
 
-from twisted.web.template import tags
-from twisted.web.template import Element, renderer, TagLoader, XMLFile
-
 from pydoctor import epydoc2stan, model
 from pydoctor.templatewriter import util
+from twisted.web.template import Element, TagLoader, XMLFile, renderer, tags
+
 
 def moduleSummary(modorpack):
-    r = tags.li(util.taglink(modorpack), ' - ', epydoc2stan.doc2stan(modorpack, summary=True)[0])
+    r = tags.li(
+        util.taglink(modorpack), ' - ',
+        epydoc2stan.format_summary(modorpack)
+        )
     if not isinstance(modorpack, model.Package):
         return r
-    contents = [m for m in modorpack.orderedcontents
+    contents = [m for m in modorpack.contents.values()
                 if m.isVisible and m.name != '__init__']
     if not contents:
         return r
@@ -74,7 +76,7 @@ def subclassesFrom(hostsystem, cls, anchors):
     if name not in anchors:
         r(tags.a(name=name))
         anchors.add(name)
-    r(util.taglink(cls), ' - ', epydoc2stan.doc2stan(cls, summary=True)[0])
+    r(util.taglink(cls), ' - ', epydoc2stan.format_summary(cls))
     scs = [sc for sc in cls.subclasses if sc.system is hostsystem and ' ' not in sc.fullName()
            and sc.isVisible]
     if len(scs) > 0:
@@ -175,7 +177,7 @@ class NameIndexPage(Element):
     def __init__(self, system):
         self.system = system
         self.initials = {}
-        for ob in self.system.orderedallobjects:
+        for ob in self.system.allobjects.values():
             if ob.isVisible:
                 self.initials.setdefault(ob.name[0].upper(), []).append(ob)
 
@@ -295,7 +297,7 @@ class UndocumentedSummaryPage(Element):
 
     @renderer
     def stuff(self, request, tag):
-        undoccedpublic = [o for o in self.system.orderedallobjects
+        undoccedpublic = [o for o in self.system.allobjects.values()
                           if o.isVisible and not hasdocstring(o)]
         undoccedpublic.sort(key=lambda o:o.fullName())
         for o in undoccedpublic:
