@@ -3,33 +3,25 @@ Tests for Sphinx integration.
 """
 from __future__ import print_function
 
+import datetime
+import io
+import string
+import zlib
 from contextlib import closing
 
 import cachecontrol
-
-import datetime
-import zlib
-
-import io
-from hypothesis import given, strategies as st, settings
-
-from twisted.python.compat import NativeStringIO
-
-from pydoctor import model
-from pydoctor import sphinx
-
 import pytest
-
 import requests
-
-import string
-
+from pydoctor import model, sphinx
 from urllib3 import HTTPResponse
 
+from hypothesis import given, settings
+from hypothesis import strategies as st
 
-class PersistentStringIO(NativeStringIO):
+
+class PersistentBytesIO(io.BytesIO):
     """
-    A custom stringIO which keeps content after file is closed.
+    A custom BytesIO which keeps content after file is closed.
     """
     def close(self):
         """
@@ -39,14 +31,7 @@ class PersistentStringIO(NativeStringIO):
 
 
 
-def make_SphinxInventory(logger=object()):
-    """
-    Return a SphinxInventory.
-    """
-    return sphinx.SphinxInventory(logger=logger, project_name='project_name')
-
-
-def make_SphinxInventoryWithLog():
+def make_SphinxInventoryWithLog(factory=sphinx.SphinxInventory):
     """
     Return a SphinxInventory with patched log.
     """
@@ -57,18 +42,18 @@ def make_SphinxInventoryWithLog():
         """
         log.append((section, msg, thresh))
 
-    inventory = make_SphinxInventory(logger=msg)
+    inventory = factory(logger=msg)
     return (inventory, log)
 
 
-def test_initialization():
+def test_writer_initialization():
     """
     Is initialized with logger and project name.
     """
     logger = object()
     name = object()
 
-    sut = sphinx.SphinxInventory(logger=logger, project_name=name)
+    sut = sphinx.SphinxInventoryWriter(logger=logger, project_name=name)
 
     assert logger is sut.info
     assert name is sut.project_name
@@ -84,8 +69,8 @@ def test_generate_empty_functional():
     log = []
     logger = lambda section, message, thresh=0: log.append((
         section, message, thresh))
-    sut = sphinx.SphinxInventory(logger=logger, project_name=project_name)
-    output = PersistentStringIO()
+    sut = sphinx.SphinxInventoryWriter(logger=logger, project_name=project_name)
+    output = PersistentBytesIO()
     sut._openFileForWriting = lambda path: closing(output)
 
     sut.generate(subjects=[], basepath='base-path')
@@ -97,7 +82,7 @@ def test_generate_empty_functional():
         )]
     assert expected_log == log
 
-    expected_ouput = """# Sphinx inventory version 2
+    expected_ouput = b"""# Sphinx inventory version 2
 # Project: some-name
 # Version: 2.0
 # The rest of this file is compressed with zlib.
@@ -110,20 +95,21 @@ def test_generateContent():
     """
     Return a string with inventory for all  targeted objects, recursive.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
     system = model.System()
-    root1 = model.Package(system, 'package1', 'docstring1')
-    root2 = model.Package(system, 'package2', 'docstring2')
-    child1 = model.Package(system, 'child1', 'docstring3', parent=root2)
+    root1 = model.Package(system, 'package1')
+    root2 = model.Package(system, 'package2')
+    child1 = model.Package(system, 'child1', parent=root2)
     system.addObject(child1)
     subjects = [root1, root2]
 
     result = sut._generateContent(subjects)
 
     expected_result = (
-        'package1 py:module -1 package1.html -\n'
-        'package2 py:module -1 package2.html -\n'
-        'package2.child1 py:module -1 package2.child1.html -\n'
+        b'package1 py:module -1 package1.html -\n'
+        b'package2 py:module -1 package2.html -\n'
+        b'package2.child1 py:module -1 package2.child1.html -\n'
         )
     assert expected_result == result
 
@@ -132,10 +118,11 @@ def test_generateLine_package():
     """
     Check inventory for package.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
 
     result = sut._generateLine(
-        model.Package('ignore-system', 'package1', 'ignore-docstring'))
+        model.Package('ignore-system', 'package1'))
 
     assert 'package1 py:module -1 package1.html -\n' == result
 
@@ -144,10 +131,11 @@ def test_generateLine_module():
     """
     Check inventory for module.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
 
     result = sut._generateLine(
-        model.Module('ignore-system', 'module1', 'ignore-docstring'))
+        model.Module('ignore-system', 'module1'))
 
     assert 'module1 py:module -1 module1.html -\n' == result
 
@@ -156,10 +144,11 @@ def test_generateLine_class():
     """
     Check inventory for class.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
 
     result = sut._generateLine(
-        model.Class('ignore-system', 'class1', 'ignore-docstring'))
+        model.Class('ignore-system', 'class1'))
 
     assert 'class1 py:class -1 class1.html -\n' == result
 
@@ -170,11 +159,12 @@ def test_generateLine_function():
 
     Functions are inside a module.
     """
-    sut = make_SphinxInventory()
-    parent = model.Module('ignore-system', 'module1', 'docstring')
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
+    parent = model.Module('ignore-system', 'module1')
 
     result = sut._generateLine(
-        model.Function('ignore-system', 'func1', 'ignore-docstring', parent))
+        model.Function('ignore-system', 'func1', parent))
 
     assert 'module1.func1 py:function -1 module1.html#func1 -\n' == result
 
@@ -185,11 +175,12 @@ def test_generateLine_method():
 
     Methods are functions inside a class.
     """
-    sut = make_SphinxInventory()
-    parent = model.Class('ignore-system', 'class1', 'docstring')
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
+    parent = model.Class('ignore-system', 'class1')
 
     result = sut._generateLine(
-        model.Function('ignore-system', 'meth1', 'ignore-docstring', parent))
+        model.Function('ignore-system', 'meth1', parent))
 
     assert 'class1.meth1 py:method -1 class1.html#meth1 -\n' == result
 
@@ -198,11 +189,12 @@ def test_generateLine_attribute():
     """
     Check inventory for attributes.
     """
-    sut = make_SphinxInventory()
-    parent = model.Class('ignore-system', 'class1', 'docstring')
+    sut = sphinx.SphinxInventoryWriter(logger=object(),
+                                       project_name='project_name')
+    parent = model.Class('ignore-system', 'class1')
 
     result = sut._generateLine(
-        model.Attribute('ignore-system', 'attr1', 'ignore-docstring', parent))
+        model.Attribute('ignore-system', 'attr1', parent))
 
     assert 'class1.attr1 py:attribute -1 class1.html#attr1 -\n' == result
 
@@ -218,10 +210,13 @@ def test_generateLine_unknown():
     When object type is uknown a message is logged and is handled as
     generic object.
     """
-    sut, log = make_SphinxInventoryWithLog()
+    sut, log = make_SphinxInventoryWithLog(
+        lambda **kwargs: sphinx.SphinxInventoryWriter(
+                                    project_name='project_name', **kwargs)
+        )
 
     result = sut._generateLine(
-        UnknownType('ignore-system', 'unknown1', 'ignore-docstring'))
+        UnknownType('ignore-system', 'unknown1'))
 
     assert 'unknown1 py:obj -1 unknown1.html -\n' == result
 
@@ -231,8 +226,8 @@ def test_getPayload_empty():
     """
     Return empty string.
     """
-    sut = make_SphinxInventory()
-    content = """# Sphinx inventory version 2
+    sut = sphinx.SphinxInventory(logger=object())
+    content = b"""# Sphinx inventory version 2
 # Project: some-name
 # Version: 2.0
 # The rest of this file is compressed with zlib.
@@ -247,26 +242,26 @@ def test_getPayload_content():
     """
     Return content as string.
     """
-    payload = 'first_line\nsecond line'
-    sut = make_SphinxInventory()
-    content = """# Ignored line
+    payload = u"first_line\nsecond line\nit's a snake: \U0001F40D"
+    sut = sphinx.SphinxInventory(logger=object())
+    content = b"""# Ignored line
 # Project: some-name
 # Version: 2.0
 # commented line.
-%s""" % (zlib.compress(payload),)
+%s""" % (zlib.compress(payload.encode('utf-8')),)
 
     result = sut._getPayload('http://base.ignore', content)
 
     assert payload == result
 
 
-def test_getPayload_invalid():
+def test_getPayload_invalid_uncompress():
     """
     Return empty string and log an error when failing to uncompress data.
     """
     sut, log = make_SphinxInventoryWithLog()
     base_url = 'http://tm.tld'
-    content = """# Project: some-name
+    content = b"""# Project: some-name
 # Version: 2.0
 not-valid-zlib-content"""
 
@@ -278,11 +273,30 @@ not-valid-zlib-content"""
         )] == log
 
 
+def test_getPayload_invalid_decode():
+    """
+    Return empty string and log an error when failing to uncompress data.
+    """
+    payload = b'\x80'
+    sut, log = make_SphinxInventoryWithLog()
+    base_url = 'http://tm.tld'
+    content = b"""# Project: some-name
+# Version: 2.0
+%s""" % (zlib.compress(payload),)
+
+    result = sut._getPayload(base_url, content)
+
+    assert '' == result
+    assert [(
+        'sphinx', 'Failed to decode inventory from http://tm.tld', -1,
+        )] == log
+
+
 def test_getLink_not_found():
     """
     Return None if link does not exists.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
 
     assert None is sut.getLink('no.such.name')
 
@@ -291,7 +305,7 @@ def test_getLink_found():
     """
     Return the link from internal state.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
     sut._links['some.name'] = ('http://base.tld', 'some/url.php')
 
     assert 'http://base.tld/some/url.php' == sut.getLink('some.name')
@@ -301,7 +315,7 @@ def test_getLink_self_anchor():
     """
     Return the link with anchor as target name when link end with $.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
     sut._links['some.name'] = ('http://base.tld', 'some/url.php#$')
 
     assert 'http://base.tld/some/url.php#some.name' == sut.getLink('some.name')
@@ -312,12 +326,12 @@ def test_update_functional():
     Functional test for updating from an empty inventory.
     """
     payload = (
-        'some.module1 py:module -1 module1.html -\n'
-        'other.module2 py:module 0 module2.html Other description\n'
+        b'some.module1 py:module -1 module1.html -\n'
+        b'other.module2 py:module 0 module2.html Other description\n'
         )
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
     # Patch URL loader to avoid hitting the system.
-    content = """# Sphinx inventory version 2
+    content = b"""# Sphinx inventory version 2
 # Project: some-name
 # Version: 2.0
 # The rest of this file is compressed with zlib.
@@ -367,7 +381,7 @@ def test_parseInventory_empty():
     """
     Return empty dict for empty input.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
 
     result = sut._parseInventory('http://base.tld', '')
 
@@ -378,7 +392,7 @@ def test_parseInventory_single_line():
     """
     Return a dict with a single member.
     """
-    sut = make_SphinxInventory()
+    sut = sphinx.SphinxInventory(logger=object())
 
     result = sut._parseInventory(
         'http://base.tld', 'some.attr py:attr -1 some.html De scription')
@@ -619,7 +633,7 @@ class TestStubCache(object):
     clearCache=st.booleans(),
     enableCache=st.booleans(),
     cacheDirectoryName=st.text(
-        alphabet=sorted(set(string.printable) - set('\/:*?"<>|\x0c\x0b\n')),
+        alphabet=sorted(set(string.printable) - set('\\/:*?"<>|\x0c\x0b\n')),
         min_size=3,             # Avoid ..
         max_size=32,            # Avoid upper length on path
     ),
